import warnings

warnings.warn(
    message=(
        "thehive4py v1.x is not maintained anymore as TheHive v3 and v4 "
        "are end of life. If you are using TheHive v5 please consider "
        "switching to thehive4py v2.x "
        "(https://github.com/TheHive-Project/TheHive4py)!"
    ),
    category=DeprecationWarning,
    stacklevel=2,
)



"""Test what happens when we try to attach files to things."""
try:
    import mock
except ImportError:
    import unittest.mock as mock

import thehive4py.models
from thehive4py.api import TheHiveApi
from thehive4py.models import CaseObservable
from io import StringIO

@mock.patch('thehive4py.api.requests.post')
def test_file_observable_named_file(mock_post):
    thehive = TheHiveApi('http://127.0.0.1:9000', 'API_KEY')
    test_id = 'AV55EOIsPQ_zDQrlj4a9'
    with mock.patch('thehive4py.models.open', mock.mock_open()):
        file_observable = CaseObservable(dataType='file',
                                         data='pic.png',
                                         tlp=1,
                                         ioc=True,
                                         tags=['thehive4py'],
                                         message='test')
        response = thehive.create_case_observable(test_id, file_observable)
        # if we just pass in the filename, the file has to be opened ...
        assert thehive4py.models.open.call_count > 0
        # ... but it can't be closed in this scope - this is the problem of #10
        # assert thehive4py.models.open.return_value.close.call_count == thehive4py.models.open.call_count
    assert mock_post.call_count == 1

@mock.patch('thehive4py.api.requests.post')
def test_file_observable_file_object(mock_post):
    thehive = TheHiveApi('http://127.0.0.1:9000', 'API_KEY')
    test_id = 'AV55EOIsPQ_zDQrlj4a9'
    our_file = mock.Mock(wraps=StringIO(u'contents of file'))
    with mock.patch('thehive4py.models.open', mock.mock_open()):
        file_observable = CaseObservable(dataType='file',
                                         data=(our_file, 'pic.png'),
                                         tlp=1,
                                         ioc=True,
                                         tags=['thehive4py'],
                                         message='test')
        response = thehive.create_case_observable(test_id, file_observable)
        # if we pass in the file object, it does not have to be opened ...
        assert thehive4py.models.open.call_count == 0
        # ... operations happen on our file ...
        assert our_file.read.call_count > 0
        assert our_file.seek.call_count > 0
        # ... and balance between opens and closes is achieved
        assert thehive4py.models.open.return_value.close.call_count == thehive4py.models.open.call_count
        assert our_file.close.call_count == 0
    assert mock_post.call_count == 1





#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""

## Overview

This is a module that defines a set of utility methods used to easily build valid search query
without the need to use JSON objects.

Query objects are used by APIs that allow searchinf for objects like:

* [thehive4py.api.find_cases](./api#thehive4py.api.TheHiveApi.find_cases)
* [thehive4py.api.find_alerts](./api#thehive4py.api.TheHiveApi.find_alerts)
* [thehive4py.api.find_tasks](./api#thehive4py.api.TheHiveApi.find_tasks)

A query is a defined by on of the criteria described below.

A query could be used like below:

```python
# Define the query
query = And(
    Eq('owner', 'admin'),
    Between('tlp', 1, 3),
    Not(Eq('status', 'Deleted')),
    Child('case_artifact', And(
        Eq('ioc', True),
        In('dataType', ['file', 'ip', 'domain'])
    ))
)

# Call find_cases method to search for all the case, sorted by descending created date, 
# and verifying the following set of conditions:
# - owned by admin
# - having tlp between GREEN and RED
# - not deleted
# - having observables of type file, ip or domain that are flagged as IOC

api.find_cases(query=query, sort=['-createdAt'], range='all')
```
"""

def Eq(field, value):
    """
    A criterion used to search for a equality. For example

    * search for TLP = 2
    * search for flag = True
    * search for title = 'Sample case'

    Arguments:
        field (value): field name
        value (Any): field value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            query = Eq('tlp', 3)
            ```
            produces
            ```json
            {"_field": "tlp", "_value": 3}
            ```
    """
    return {'_field': field, '_value': value}


def Gt(field, value):
    """
    A criterion used to search for a field greater than a certain value. For example

    * search for TLP > 2
    * search for customFields.cvss > 4.5
    * search for date > now

    Arguments:
        field (value): field name
        value (Any): field value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            query = Gt('tlp', 1)
            ```
            produces
            ```json
            {"_gt": {"tlp": 1}}
            ```
    """
    return {'_gt': {field: value}}


def Gte(field, value):
    """
    A criterion used to search for a field greater or equal than a certain value. For example

    * search for TLP >= 2
    * search for customFields.cvss >= 4.5
    * search for date >= now

    Arguments:
        field (value): field name
        value (Any): field value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            query = Gte('tlp', 1)
            ```
            produces
            ```json
            {"_gte": {"tlp": 1}}
            ```
    """
    return {'_gte': {field: value}}


def Lt(field, value):
    """
    A criterion used to search for a field less than a certain value. For example

    * search for TLP < 2
    * search for customFields.cvss < 4.5
    * search for date < now

    Arguments:
        field (value): field name
        value (Any): field value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            query = Lt('tlp', 3)
            ```
            produces
            ```json
            {"_lt": {"tlp": 3}}
            ```
    """
    return {'_lt': {field: value}}


def Lte(field, value):
    """
    A criterion used to search for a field less or equal than a certain value. For example

    * search for TLP <= 2
    * search for customFields.cvss <= 4.5
    * search for date <= now

    Arguments:
        field (value): field name
        value (Any): field value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            query = Lte('tlp', 3)
            ```
            produces
            ```json
            {"_lte": {"tlp": 3}}
            ```
    """
    return {'_lte': {field: value}}


def And(*criteria):
    """
    A criterion used to search for records that verfies all the specified criteria. For example

    * search for observables flagged as IOC and having TLP <= 2
    * search for closed cases related to customFields.customer "Company" and having a WHITE tlp

    Arguments:
        criteria (Array): A set of criteria, can be any operator defined in the query module (`Eq`, `In`, `Not`, `And`...)

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for cases with customer custom field set to 'Company', having a TLP less than Amber and are resolved
            query = And(
                Eq('customFields.customer.string', 'Company'),
                Lte('tlp', 2),
                Eq('status', 'Resolved')
            )
            ```
            produces
            ```json
            {
                "_and": [
                    {
                        "_field": "customFields.customer.string",
                        "_value": "Company"
                    },
                    {
                        "_lte": {
                            "tlp": 2
                        }
                    },
                    {
                        "_field": "status",
                        "_value": "Resolved"
                    }
                ]
            }
            ```
    """
    return {'_and': criteria}


def Or(*criteria):
    """
    A criterion used to search for records that verfies one of the specified criteria. For example

    * search for observables of type domain or fqdn
    * search for cases assigned to a given user or have a specific tag

    Arguments:
        criteria (Array): A set of criteria, can be any operator defined in the query module (`Eq`, `In`, `Not`, `And`...)

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for cases assigned to jdoe or are flagged and open
            query = Or(
                Eq('owner', 'jdoe'),
                And(
                    Eq('status', 'Open'),
                    Eq('flag', True),
                )
            )
            ```
            produces
            ```json
            {
                "_or": [
                    {
                        "_field": "owner",
                        "_value": "jdoe"
                    },
                    {
                        "_and": [
                            [
                                {
                                    "_field": "status",
                                    "_value": "Open"
                                },
                                {
                                    "_field": "flag",
                                    "_value": true
                                }
                            ]
                        ]
                    }
                ]
            }
            ```
    """
    return {'_or': criteria}


def Not(criterion):
    """
    A criterion used to search for records that verfies the opposite of the specified criterion. For example

    * search for observables not marked as ioc
    * search for cases not having a MISP tag

    Arguments:
        crietrion (Any): The base criterion to use for negation

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for cases assigned to jdoe or are flagged and open
            query = Not(Eq('ioc', True))
            ```
            produces
            ```json
            {
                "_not": {
                    "_field": "ioc",
                    "_value": false
                }
            }
            ```
    """
    return {'_not': criterion}


def In(field, values):
    """
    A criterion used to search for records where `field` has one of the `values`. For example

    * search for observables of type domain, fqdn, ip
    * search for cases tagged as malspam, or phising

    Arguments:
        field (str): field name
        values (Array): A set of values the field must be in

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for observables of one of the following types: domain, fqdn, ip
            query = In('dataType', ['domain', 'fqdn', 'ip'])
            ```
            produces
            ```json
            {
                "_in": {
                    "_field": "dataType",
                    "_values": [
                        "domain",
                        "fqdn",
                        "ip"
                    ]
                }
            }
            ```
    """
    return {'_in': {'_field': field, '_values': values}}


def Contains(field):
    """
    A criterion used to search for records where `field` id defined. For example
    
    * search for cases that have the custom field `customer`

    Arguments:
        field (str): field name        

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for cases having a 'customer' custom field
            query = Contains('customFields.customer')
            ```
            produces
            ```json
            {
                "_contains": "customFields.customer"
            }
            ```
    """
    return {'_contains': field}


def Id(id):
    """
    A criterion used to search for records by id. For example
    
    * search for a case by its id
    * search for an alert by its id

    Arguments:
        id (str): the id's value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for as case by id
            query = Id('1234545643')
            ```
            produces
            ```json
            {
                "_id": "1234545643"
            }
            ```
    """
    return {'_id': id}


def Between(field, from_value, to_value):
    """
    A criterion used to search for records having `field`'s value included in a range defined by `from_value` and `to_value`. 
    This is an idea criterion to seahrch using date conditions. For example
    
    * search for cases created between two dates
    * search for alerts with cvss custom field greater that 2 and lesser than 9

    Arguments:
        field (str): field name
        from_value (Number): Lower limit of the range
        to_value (Number): Higher limit of the range

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for alerts with cvss custom field greater that 2 and lesser than 9
            query = Between('customFields.cvss.float', 2, 9)
            ```
            produces
            ```json
            {
                "_between": {
                    "_field": "customFields.cvss.float",
                    "_from": 2,
                    "_to": 9
                }
            }
            ```
    """
    return {'_between': {'_field': field, '_from': from_value, '_to': to_value}}


def ParentId(tpe, id):
    """
    A criterion used to search for records by their parent's id. For example
    
    * search for observables by case id
    * search for tasks by case id
    * search for logs by task id
    * search for jobs by observable id

    Arguments:
        tpe (str): class name of the parent: `case`, `case_task`, `case_artifact`...
        id (str): the parent id's value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for tasks of a case by id
            query = ParentId('case', '1234545643')
            ```
            produces
            ```json
            {
                "_parent": {
                    "_type": "case",
                    "_id": "1234545643"
                }
            }
            ```
    """
    return {'_parent': {'_type': tpe, '_id': id}}


def Parent(tpe, criterion):
    """
    A criterion used to search for records by filtering the parents using a criterion. For example
    
    * search for observables of TLP:RED cases
    * search for logs of tasks called 'Communication'

    Arguments:
        tpe (str): class name of the parent: `case`, `case_task`, `case_artifact`...
        criterion (Any): Any criterion defined by functions from the query module

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for tasks belonging to open cases with TLP=RED
            query = Parent('case', And(
                Eq('status', 'Open'),
                Eq('tlp', 3)
            ))
            ```
            produces
            ```json
            {
                "_parent": {
                    "_type": "case",
                    "_query": {
                        "_and": [
                            [
                                {
                                    "_field": "status",
                                    "_value": "Open"
                                },
                                {
                                    "_field": "tlp",
                                    "_value": 3
                                }
                            ]
                        ]
                    }
                }
            }
            ```
    """
    return {'_parent': {'_type': tpe, '_query': criterion}}


def Child(tpe, criterion):
    """
    A criterion used to search for records by applying a filter on their children using a criterion. For example
    
    * search for cases having file observables
    * search for cases having a specific ip observable

    Arguments:
        tpe (str): class name of the child: `case_task`, `case_artifact`...
        criterion (Any): Any criterion defined by functions from the query module

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for cases having iocs of type file
            query = Child('case_artifact', And(
                Eq('dataType', 'file'),
                Eq('ioc', True)
            ))
            ```
            produces
            ```json
            {
                "_child": {
                    "_type": "case_artifact",
                    "_query": {
                        "_and": [
                            [
                                {
                                    "_field": "dataType",
                                    "_value": "file"
                                },
                                {
                                    "_field": "ioc",
                                    "_value": true
                                }
                            ]
                        ]
                    }
                }
            }
            ```
    """
    return {'_child': {'_type': tpe, '_query': criterion}}


def Type(tpe):
    """
    A criterion used to search for records of the type defined by `tpe`. For example
    
    * search for objects of type 'audit'
    * search for objects of type 'alert'

    Arguments:
        tpe (str): object type's name as defined in TheHive: `all`, `case`, `case_task`, `case_task_log`, `case_artifact`, `alert`, `case_artifact_job`, `audit`

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for alerts
            query = Type('alert')
            ```
            produces
            ```json
            {
                "_type": "alert"
            }
            ```
    """
    return {'_type': tpe}


def String(query_string):
    """
    A criterion used to search for objects using Elasticsearch querystring syntax. For example
    
    * search for case using `title:misp AND tlp:2`

    Arguments:
        tpe (str): object type's name as defined in TheHive: 
            
            Possible values: `all`, `case`, `case_task`, `case_task_log`, `case_artifact`, `alert`, `case_artifact_job`, `audit`

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Query to search for casee with TLP:AMBER and having the word 'misp' on the title
            query = String('title:misp AND tlp:2')
            ```
            produces
            ```json
            {
                "_string": "title:misp AND tlp:2"
            }
            ```
    
    !!! Warning
        This criterion is deprecated and won't be ported to TheHive 4

    !!! Warning
        This criterion is available in TheHive 3 ONLY
    """
    return {'_string': query_string}


def Like(field, value):
    """
    A criterion used to search for objects having a text field's value like the specified `value`. 
    It's a wildcard operator when the searched value must specify asteriscs. For example:

    * search for cases where title is like `*malspam*`
    * search for observable where description contains the text `*malware*`

    Arguments:
        field (value): field name
        value (Any): searched value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Search for tasks where title contains 'Communication'
            query = Like('title', '*Communication*')
            ```
            produces
            ```json
            {
                "_like": {
                    "_field": "title",
                    "_value": "*Communication*"
                }
            }
            ```

    !!! Note
        If the `*` are not specified, the exact same text will be searched for.
        `Like('title', 'MISP')` will search for titles equal to `'MISP'`

    """
    return {'_like': {'_field': field, '_value': value}}


def StartsWith(field, value):
    """
    A criterion used to search for objects having a text field's value start with `value`. 
    It's a wildcard operator that adds the `*` if not specified, at the end of `value`. For example:

    * search for cases having a tag starting with `malspam`
    * search for filename observables starting with `dridex`

    Arguments:
        field (value): field name
        value (Any): searched value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Search for tasks where title stats with 'Communication'
            query = StartsWith('title', 'Communication')
            ```
            produces
            ```json
            {
                "_wildcard": {
                    "_field": "title",
                    "_value": "Communication*"
                }
            }
            ```
    """
    if not value.endswith('*'):
        value = value + '*'

    return {'_wildcard': {'_field': field, '_value': value}}


def EndsWith(field, value):
    """
    A criterion used to search for objects having a text field's value end with `value`. 
    It's a wildcard operator that adds the `*` if not specified, at the beginning of `value`. For example:

    * search for filename observables ending with `.exe`

    Arguments:
        field (value): field name
        value (Any): searched value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Search for tasks where title stats with 'Communication'
            query = EndsWith('data', '.png')
            ```
            produces
            ```json
            {
                "_wildcard": {
                    "_field": "data",
                    "_value": "*.png"
                }
            }
            ```
    """
    if not value.startswith('*'):
        value = '*' + value

    return {'_wildcard': {'_field': field, '_value': value}}


def ContainsString(field, value):
    """
    A criterion used to search for objects having a text field's value like the specified `value`. 
    It's a wildcard operator that wraps the searched value with asteriscs. It operates the same way a the `Like` criterion For example:

    * search for cases where title is like `*malspam*`
    * search for observable where description contains the text `*malware*`

    Arguments:
        field (value): field name
        value (Any): searched value

    Returns:
        dict: JSON repsentation of the criterion
            ```python
            # Search for tasks where title contains 'Communication'
            query = ContainsString('title', 'Communication')
            ```
            produces
            ```json
            {
                "_wildcard": {
                    "_field": "title",
                    "_value": "*Communication*"
                }
            }
            ```
    """
    if not value.endswith('*'):
        value = value + '*'

    if not value.startswith('*'):
        value = '*' + value

    return {'_wildcard': {'_field': field, '_value': value}}



class TheHiveException(Exception):
    """
    Base class for TheHive exceptions
    """
    pass


class CaseException(TheHiveException):
    """
    Exception raised by failure of API calls related to `Case` handling
    """
    pass


class CaseTaskException(CaseException):
    """
    Exception raised by failure of API calls related to `Case Task` handling
    """
    pass


class CaseTaskLogException(CaseTaskException):
    """
    Exception raised by failure of API calls related to `Case Task Log` handling
    """
    pass


class CaseObservableException(CaseException):
    """
    Exception raised by failure of API calls related to `Case Observable` handling
    """
    pass


class ObservableException(TheHiveException):
    """
    Exception raised by failure of API calls related to `Observable` handling
    """
    pass


class AlertException(TheHiveException):
    """
    Exception raised by failure of API calls related to `Alert` handling
    """
    pass


class AlertArtifactException(CaseException):
    """
    Exception raised by failure of API calls related to `Alert Artifact` handling
    """
    pass


class CaseTemplateException(TheHiveException):
    """
    Exception raised by failure of API calls related to `Case Template` handling
    """
    pass


class CustomFieldException(TheHiveException):
    """
    Exception raised by failure of API calls related to `Custom Fields` handling
    """
    pass



from thehive4py.query import *


def test_like():
    query = Like('name', 'thehive')

    assert '_like' in query
    assert '_field' in query['_like']
    assert '_value' in query['_like']

    assert query['_like']['_field'] == 'name'
    assert query['_like']['_value'] == 'thehive'


def test_startswith():
    query = StartsWith('name', 'thehive')

    assert '_wildcard' in query
    assert '_field' in query['_wildcard']
    assert '_value' in query['_wildcard']

    assert query['_wildcard']['_field'] == 'name'
    assert query['_wildcard']['_value'] == 'thehive*'


def test_endwith():
    query = EndsWith('name', 'thehive')

    assert '_wildcard' in query
    assert '_field' in query['_wildcard']
    assert '_value' in query['_wildcard']

    assert query['_wildcard']['_field'] == 'name'
    assert query['_wildcard']['_value'] == '*thehive'


def contains_string():
    query = ContainsString('name', 'thehive')

    assert '_wildcard' in query
    assert '_field' in query['_wildcard']
    assert '_value' in query['_wildcard']

    assert query['_wildcard']['_field'] == 'name'
    assert query['_wildcard']['_value'] == '*thehive*'
# query = And(Eq('status', 'Ok'), Eq('tlp', 2))
#
#
#
# print(json.dumps(query, indent=4, sort_keys=True))
#
# query = And(Parent('case', Id('XXXXXXXXXXXXXXx')), Eq('status', 'Ok'))
# print(json.dumps(query, indent=4, sort_keys=True))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import datetime
import json
import os
import time

from enum import Enum
import magic
import requests
from future.utils import raise_with_traceback

from thehive4py.exceptions import TheHiveException, CaseException


class Version(Enum):
    """
    Enumeration representing a version used to specify the version of TheHive instance

    Possible values: THEHIVE_3, THEHIVE_4
    """
    THEHIVE_3 = 3
    THEHIVE_4 = 4


class Tlp(Enum):
    """
    Enumeration representing TLP, used in cases, observables and alerts

    Possible values: WHITE, GREEN, AMBER, RED
    """
    WHITE = 0
    GREEN = 1
    AMBER = 2
    RED = 3


class Pap(Enum):
    """
    Enumeration representing PAP, used in cases, observables and alerts (TheHive 4 only)

    Possible values: WHITE, GREEN, AMBER, RED
    """
    WHITE = 0
    GREEN = 1
    AMBER = 2
    RED = 3


class Severity(Enum):
    """
    Enumeration representing severity, used in cases and alerts

    Possible values: LOW, MEDIUM, HIGH, CRITICAL
    """
    LOW = 1
    MEDIUM = 2
    HIGH = 3
    CRITICAL = 4


class CaseStatus(Enum):
    """
    Enumeration representing case statuses

    Possible values: OPEN, RESOLVED, DELETED, DUPLICATE
    """
    OPEN = 'Open'
    RESOLVED = 'Resolved'
    DELETED = 'Deleted'
    DUPLICATE = 'Duplicate'


class TaskStatus(Enum):
    """
    Enumeration representing task statuses

    Possible values: WAITING, INPROGRESS, COMPLETED, CANCEL
    """
    WAITING = 'Waiting'
    INPROGRESS = 'InProgress'
    COMPLETED = 'Completed',
    CANCEL = 'Cancel'


class CustomJsonEncoder(json.JSONEncoder):
    """
    Custom JSON encoder class that takes into account [thehive4py.models.JSONSerializable][] instances and 
    `datetime.datetime` objects
    """
    def default(self, o):
        """
        Method to serialize [thehive4py.models.JSONSerializable][] objects.

        Used by [thehive4py.models.JSONSerializable.jsonify][] method
        """
        if isinstance(o, JSONSerializable):
            return o.__dict__
        elif isinstance(o, datetime.datetime):
            return o.timestamp()
        else:
            return json.JSONEncoder.default(self, o)


class JSONSerializable(object):
    """
    Abstract class of all the models classes. 
    
    It defines utility methods called `jsonify` used to get a model object's JSON representation
    """

    def jsonify(self, excludes=[]):
        """
        A method that returns a stringyfied JSON representing a model object

        Arguments:
            excludes (str[]): list of fields to exclude from the returned JSON object.

        Returns:
            str: the JSON string of the object.
        """
        data = self.__dict__

        for ex in excludes:
            if ex in data:
                del data[ex]

        return json.dumps(data, sort_keys=True, indent=4, cls=CustomJsonEncoder)

    def attr(self, attributes, name, default, error=None):
        is_required = error is not None

        if is_required and name not in attributes:
            raise_with_traceback(ValueError(error))
        else:
            return attributes.get(name, default)


class CustomFieldHelper(object):
    """
    CustomFieldHelper
    """
    def __init__(self):
        self.fields = {}

    def __add_field(self, type, name, value):
        custom_field = dict()
        custom_field['order'] = len(self.fields)
        custom_field[type] = value
        self.fields[name] = custom_field

    def add_date(self, name, value):
        """
        Add a custom field of type `date`.

        Arguments:
            name (str): name of the custom field
            value (int): number of milliseconds representing a timestamp (Example: int(time.time())*1000)
        
        """
        self.__add_field('date', name, value)
        return self

    def add_string(self, name, value):
        """
        Add a custom field of type `string`.

        Arguments:
            name (str): name of the custom field
            value (str): value of the custom field
        
        """
        self.__add_field('string', name, value)
        return self

    def add_boolean(self, name, value):
        """
        Add a custom field of type `bool`.

        Arguments:
            name (str): name of the custom field
            value (bool): True or False, value of the custom field
        
        """
        self.__add_field('boolean', name, value)
        return self

    def add_number(self, name, value):
        """
        Add a custom field of type `number`.

        Arguments:
            name (str): name of the custom field
            value (number): value of the custom field
        
        !!! Warning
            This is method that work for TheHive 3 ONLY
        """
        self.__add_field('number', name, value)
        return self

    def add_integer(self, name, value):
        """
        Add a custom field of type `integer`.

        Arguments:
            name (str): name of the custom field
            value (int): value of the custom field
        
        !!! Warning
            This is method that work for TheHive 4 ONLY
        """
        self.__add_field('integer', name, value)
        return self

    def add_float(self, name, value):
        """
        Add a custom field of type `float`.

        Arguments:
            name (str): name of the custom field
            value (float): value of the custom field
        
        !!! Warning
            This is method that work for TheHive 4 ONLY
        """
        self.__add_field('float', name, value)
        return self

    def build(self):
        """
        Builds the custom field value dict as expected by TheHive, 
        maintining the order of the fields, specified by `order`

        Returns:
            dict: A json representation of the custom fields map
        """
        return self.fields


class CustomField(object):
    """
    Model class describing a custom field as defined in TheHive

    Arguments:
        name (str): name of the custom field
        reference (str): internal reference name
        description (str): description of the custom field
        type (Enum): type of the field, possible values are `string`, `boolean`, `number`, `date`, `integer`, `float`
        options (Any[]): list of possible values for the field
        mandatory (bool): True if the field is mandatory
    """

    def __init__(self, **attributes):
        self.name = attributes.get('name', None)
        self.reference = attributes.get('name', None)
        self.description = attributes.get('description', None)
        self.type = attributes.get('type', None)
        self.options = attributes.get('options', [])
        self.mandatory = attributes.get('mandatory', False)


class Case(JSONSerializable):
    """
    Model class describing a case as defined in TheHive

    Arguments:
        id (str): Case's id. Default: None
        title (str): Case's description. Default: None
        description (str): Case's description. Default: None
        tlp (Enum): Case's TLP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        pap (Enum): Case's PAP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        severity (Enum): Case's severity: `1`, `2`, `3`, `4` for `LOW`, `MEDIUM`, `HIGH`, `CRTICAL`. Default: `2`
        flag (bool): Case's flag, `True` to mark the case as important. Default: `False`
        tags (str[]): List of case tags. Default: `[]`
        startDate (datetime): Case's start date, the date the case occured. Default: `Now()`
        template (str): Case template's name. If specified then the case is created using the given template. Default: `None`
        owner (str): Case's assignee. Default: `None`
        metrics (JSON): Case metrics collection. A JSON object where keys are defining metric name, and values are defining metric value. Default: `{}`
        customFields (CustomField[]): A set of CustomField instances, or the result of a CustomFieldHelper.build() method. Default: `{}`
        tasks (JSON[] / CaseTask[]): Set of taks, defined either as JSON objects or CaseTask instances
        json (JSON): If the field is not equal to None, the case is instantiated using the JSON value instead of the arguements

    !!! Warning
        The `metrics` field is available in TheHive 3 only
    """
    def __init__(self, **attributes):
        defaults = {
            'id': None,
            'title': None,
            'description': None,
            'tlp': Tlp.AMBER.value,
            'pap': Pap.AMBER.value,
            'severity': Severity.MEDIUM.value,
            'flag': False,
            'tags': [],
            'startDate': int(time.time()) * 1000,
            'metrics': {},
            'customFields': {},
            'tasks': [],
            'template': None,
            'owner': None
        }

        if attributes.get('json', False):
            attributes = attributes['json']

        is_from_template = attributes.get('template', False)
        if is_from_template:
            defaults['template'] = attributes['template']

        self.id = attributes.get('id', None)
        self.title = attributes.get('title', None)
        self.description = attributes.get('description', defaults['description'])
        self.tlp = attributes.get('tlp', defaults['tlp'])
        self.pap = attributes.get('pap', defaults['pap'])
        self.severity = attributes.get('severity', defaults['severity'])
        self.flag = attributes.get('flag', defaults['flag'])
        self.tags = attributes.get('tags', defaults['tags'])
        self.startDate = attributes.get('startDate', defaults['startDate'])
        self.metrics = attributes.get('metrics', defaults['metrics'])
        self.customFields = attributes.get('customFields', defaults['customFields'])
        self.template = attributes.get('template', defaults['template'])
        self.owner = attributes.get('owner', defaults['owner'])

        tasks = attributes.get('tasks', defaults['tasks'])
        self.tasks = []
        for task in tasks:
            if type(task) == CaseTask:
                self.tasks.append(task)
            else:
                self.tasks.append(CaseTask(json=task))


class CaseHelper:
    """
    Provides helper methods for interacting with instances of the Case class.
    """
    def __init__(self, thehive):
        """
        Initialize a CaseHelper instance.
        :param thehive: A TheHiveApi instance.

        """
        self._thehive = thehive

    def __call__(self, id):
        """
        Return an instance of Case with the given case ID.
        :param id: ID of a case to retrieve.

        """
        response = self._thehive.get_case(id)

        # Check for failed authentication
        if response.status_code == requests.codes.unauthorized:
            raise TheHiveException("Authentication failed")

        if response.status_code == requests.codes.not_found:
            raise CaseException("Case {} not found".format(id))

        if self.status_ok(response.status_code):
            data = response.json()
            case = Case(json=data)

            # Add attributes that are not added by the constructor
            case.id = data.get('id', None)
            case.owner = data.get('owner', None)
            case.caseId = data.get('caseId', None)
            case.status = data.get('status', None)
            case.createdAt = data.get('createdAt', None)
            case.createdBy = data.get('createdBy', None)
            case.updatedAt = data.get('updatedAt', None)
            case.updatedBy = data.get('updatedBy', None)

            return case

    def create(self, title, description, **kwargs):
        """
        Create an instance of the Case class.
        :param title: Case title.
        :param description: Case description.
        :param kwargs: Additional arguments.

        :return: The created instance.

        """
        case = Case(title=title, description=description, **kwargs)
        response = self._thehive.create_case(case)

        # Check for failed authentication
        if response.status_code == requests.codes.unauthorized:
            raise TheHiveException("Authentication failed")

        if self.status_ok(response.status_code):
            return self(response.json()['id'])
        else:
            raise CaseException("Server returned {}: {}".format(response.status_code, response.text))

    def update(self, case_id, **attributes):
        """
        Update a case.
        :param case_id: The ID of the case to update
        :param attributes: key=value pairs of case attributes to update (field=new_value)

        :return: The created instance.
        """

        response = self._thehive.do_patch("/api/case/{}".format(case_id), **attributes)

        if response.status_code == requests.codes.unauthorized:
            raise TheHiveException("Authentication failed")

        if self.status_ok(response.status_code):
            return self(response.json()['id'])
        else:
            raise CaseException("Server returned {}: {}".format(response.status_code, response.text))

    @staticmethod
    def status_ok(status_code):
        """Check whether a status code is OK"""
        OK_STATUS_CODES = [200, 201]
        return status_code in OK_STATUS_CODES


class CaseTask(JSONSerializable):
    """
    Model class describing a case task as defined in TheHive

    Arguments:
        id (str): Task's id. Default: None
        title (str): Task's description. Default: None
        description (str): Task's description. Default: None
        status (Enum): Task's status: `Waiting`, `InProgress`, `Cancel`, `Completed`. Default: `Waiting`
        flag (bool): Task's flag, `True` to mark the Task as important. Default: `False`
        startDate (datetime): Task's start date, the date the task started at. Default: `None`
        owner (str): Task's assignee. Default: `None`
        json (JSON): If the field is not equal to None, the Task is instantiated using the JSON value instead of the arguements
    """

    def __init__(self, **attributes):
        if attributes.get('json', False):
            attributes = attributes['json']

        self.id = attributes.get('id', None)
        self.title = attributes.get('title', None)
        self.status = attributes.get('status', TaskStatus.WAITING.value)
        self.flag = attributes.get('flag', False)
        self.description = attributes.get('description', None)
        self.owner = attributes.get('owner', None)
        self.startDate = attributes.get('startDate', None)
        self.group = attributes.get('group', None)


class CaseTaskLog(JSONSerializable):
    """
    Model class describing a case task log as defined in TheHive

    Arguments:
        id (str): Log's id. Default: None
        message (str): Log's description. Default: None
        file (str): Log attachment's path. If defined, the task log is created and the file is attached to it. Default: None
        json (JSON): If the field is not equal to None, the Task is instantiated using the JSON value instead of the arguements
    """
    def __init__(self, **attributes):
        if attributes.get('json', False):
            attributes = attributes['json']

        self.id = attributes.get('id', None)
        self.message = attributes.get('message', None)
        self.file = attributes.get('file', None)

        if self.file is not None:
            if isinstance(self.file, tuple):
                file_object, filename = self.file
            else:
                filename = self.file
                # we are opening this here, but we can't close it
                # because it gets passed into requests.post. this is
                # the substance of issue #10.
                file_object = open(self.file, 'rb')

            mime = magic.Magic(mime=True).from_buffer(file_object.read())
            file_object.seek(0)
            self.attachment = {'attachment': (filename, file_object, mime)}
            

class CaseTemplate(JSONSerializable):
    """
    Model class describing a case template as defined in TheHive

    Arguments:
        id (str): Template's id. Default: None
        titlePrefix (str): Template's title prefix. Default: None
        description (str): Template's description. Default: None
        tlp (Enum): Template's TLP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        pap (Enum): Template's PAP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        severity (Enum): Template's severity: `1`, `2`, `3`, `4` for `LOW`, `MEDIUM`, `HIGH`, `CRTICAL`. Default: `2`
        flag (bool): Template's flag, `True` to mark the case as important when created from a template. Default: `False`
        tags (str[]): List of template tags. Default: `[]`
        metrics (JSON): Template metrics collection. A JSON object where keys are defining metric name, and values are defining metric value. Default: `{}`
        customFields (CustomField[]): A set of CustomField instances, or the result of a CustomFieldHelper.build() method. Default: `{}`
        tasks (JSON[] / CaseTask[]): Set of taks, defined either as JSON objects or CaseTask instances
        json (JSON): If the field is not equal to None, the template is instantiated using the JSON value instead of the arguements

    !!! Warning
        The `metrics` field is available in TheHive 3 only
    """

    def __init__(self, **attributes):
        if attributes.get('json', False):
            attributes = attributes['json']

        self.id = attributes.get('id', None)
        self.name = attributes.get('name', None)
        self.id = attributes.get('id', None)
        self.titlePrefix = attributes.get('titlePrefix', None)
        self.description = attributes.get('description', None)
        self.severity = attributes.get('severity', Severity.MEDIUM.value)
        self.flag = attributes.get('flag', False)
        self.tlp = attributes.get('tlp', Tlp.AMBER.value)
        self.pap = attributes.get('pap', Pap.AMBER.value)
        self.tags = attributes.get('tags', [])
        self.metrics = attributes.get('metrics', {})
        self.customFields = attributes.get('customFields', {})

        tasks = attributes.get('tasks', [])
        self.tasks = []
        for task in tasks:
            if type(task) == CaseTask:
                self.tasks.append(task)
            else:
                self.tasks.append(CaseTask(json=task))


class CaseObservable(JSONSerializable):
    """
    Model class describing a case observable as defined in TheHive

    Arguments:
        id (str): Observable's id. Default: None
        dataType (str): Observable's type, must be a valid type, one of the defined data types in TheHive. Default: None
        message (str): Observable's description. Default: None
        tlp (Enum): Case's TLP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        pap (Enum): Case's PAP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        ioc (bool): Observable's ioc flag, `True` to mark an observable as IOC. Default: `False`
        sighted (bool): Observable's sighted flag, `True` to mark the observable as sighted. Default: `False`
        ignoreSimilarity (bool): Observable's similarity ignore flag. `True`to ignore the observable during similarity computing
        tags (str[]): List of observable tags. Default: `[]`
        data (str | (file, str)): Observable's data:

            - If the `dataType` field is set to `file`, then there are two options:

                * `data` must be equal to a string representing the file's path
                * `data` must be equal to Tuple composed by an in memory file object, and the file name

            - Otherwise, the `data` value is the observable's value
        json (JSON): If the field is not equal to None, the observable is instantiated using the JSON value instead of the arguements

    !!! Warning
        At least, one of `tags` or `message` are required. You cannot create an observable without specifying one of those fields

    !!! Warning
        `ignoreSimilarity` attribute is available in TheHive 4 ONLY
    """

    def __init__(self, **attributes):
        if attributes.get('json', False):
            attributes = attributes['json']

        self.id = attributes.get('id', None)
        self.dataType = attributes.get('dataType', None)
        self.message = attributes.get('message', None)
        self.tlp = attributes.get('tlp', Tlp.AMBER.value)
        self.tags = attributes.get('tags', [])
        self.ioc = attributes.get('ioc', False)
        self.sighted = attributes.get('sighted', False)
        self.ignoreSimilarity = attributes.get('ignoreSimilarity', False)

        data = attributes.get('data', None)
        if self.dataType == 'file':
            if isinstance(data, tuple):
                file_object, filename = data
            else:
                filename = data
                # we are opening this here, but we can't close it
                # because it gets passed into requests.post. this is
                # the substance of issue #10.
                file_object = open(filename, 'rb')
            mime = magic.Magic(mime=True).from_buffer(file_object.read())
            file_object.seek(0)
            self.data = [{'attachment': (filename, file_object, mime)}]
        else:
            self.data = data


class Alert(JSONSerializable):
    """
    Model class describing an alert as defined in TheHive

    Arguments:
        id (str): Alert's id. Default: None
        tlp (Enum): Alert's TLP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        pap (Enum): Alert's PAP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2` (TheHive 4 ONLY)
        severity (Enum): Alert's severity: `1`, `2`, `3`, `4` for `LOW`, `MEDIUM`, `HIGH`, `CRTICAL`. Default: `2`
        date (datetime): Alert's occur date. Default: `Now()`
        tags (str[]): List of alert tags. Default: `[]`

        title (str): Alert's description. Default: None
        type (str): Alert's type. Default: None
        source (str): Alert's source. Default: None
        sourceRef (str): Alert's source reference. Used to specify the unique identifier of the alert. Default: None
        externalLink (str): Alert's external link. Used to easily navigate to the source of the alert. Default: None
        description (str): Alert's description. Default: None
        customFields (CustomField[]): A set of CustomField instances, or the result of a CustomFieldHelper.build() method. Default: `{}`

        caseTemplate (str): Alert template's name. Default: `None`

        json (JSON): If the field is not equal to None, the Alert is instantiated using the JSON value instead of the arguements

    !!! Warning
            `pap`, `externalLink` attributes are available in TheHive 4 ONLY
    """

    def __init__(self, **attributes):
        if attributes.get('json', False):
            attributes = attributes['json']

        self.id = attributes.get('id', None)
        self.tlp = attributes.get('tlp', Tlp.AMBER.value)
        self.pap = attributes.get('pap', Pap.AMBER.value)
        self.severity = attributes.get('severity', Severity.MEDIUM.value)
        self.date = attributes.get('date', int(time.time()) * 1000)
        self.tags = attributes.get('tags', [])
        self.caseTemplate = attributes.get('caseTemplate', None)
        self.externalLink = attributes.get('externalLink', None)

        self.title = self.attr(attributes, 'title', None, 'Missing alert title')
        self.type = self.attr(attributes, 'type', None, 'Missing alert type')
        self.source = self.attr(attributes, 'source', None, 'Missing alert source')
        self.sourceRef = self.attr(attributes, 'sourceRef', None, 'Missing alert reference')
        self.description = self.attr(attributes, 'description', None, 'Missing alert description')
        self.customFields = self.attr(attributes, 'customFields', {})

        artifacts = attributes.get('artifacts', [])
        self.artifacts = []
        for artifact in artifacts:
            if type(artifact) == AlertArtifact:
                self.artifacts.append(artifact.as_base64())
            else:
                self.artifacts.append(AlertArtifact(json=artifact).as_base64())


class AlertArtifact(JSONSerializable):
    """
    Model class describing a alert observable as defined in TheHive

    Arguments:
        dataType (str): Observable's type, must be a valid type, one of the defined data types in TheHive. Default: None
        message (str): Observable's description. Default: None
        tlp (Enum): Case's TLP: `0`, `1`, `2`, `3` for `WHITE`, `GREEN`, `AMBER`, `RED`. Default: `2`
        ioc (bool): Observable's ioc flag, `True` to mark an observable as IOC. Default: `False`
        sighted (bool): Observable's sighted flag, `True` to mark the observable as sighted. Default: `False`
        ignoreSimilarity (bool): Observable's similarity ignore flag. `True`to ignore the observable during similarity computing
        tags (str[]): List of observable tags. Default: `[]`
        data (str | (file, str)): Observable's data:

            - If the `dataType` field is set to `file`, then there are two options:

                * `data` must be equal to a string representing the file's path
                * `data` must be equal to Tuple composed by an in memory file object, and the file name

            - Otherwise, the `data` value is the observable's value
        json (JSON): If the field is not equal to None, the observable is instantiated using the JSON value instead of the arguements

    !!! Warning
        `ignoreSimilarity` attribute is available in TheHive 4 ONLY
    """

    def __init__(self, **attributes):
        if attributes.get('json', False):
            attributes = attributes['json']

        self.dataType = attributes.get('dataType', None)
        self.message = attributes.get('message', None)
        self.tlp = attributes.get('tlp', Tlp.AMBER.value)
        self.tags = attributes.get('tags', [])
        self.ioc = attributes.get('ioc', False)
        self.sighted = attributes.get('sighted', False)

        if 'ignoreSimilarity' in attributes:
            self.ignoreSimilarity = attributes.get('ignoreSimilarity', False)

        data = attributes.get('data', None)
        if self.dataType != 'file':
            self.data = data
        else:
            if data is not None:
                if isinstance(data, tuple):
                    file_object, filename = data
                else:
                    filename = data
                    # we are opening this here, but we can't close it
                    # because it gets passed into requests.post. this is
                    # the substance of issue #10.
                    file_object = open(filename, 'rb')

                mime = magic.Magic(mime=True).from_buffer(file_object.read())
                file_object.seek(0)

                self.data = {'attachment': (filename, file_object, mime)}
            else:
                self.attachment = attributes.get('attachment', None)

    def as_base64(self):
        if 'data' in self.__dict__ and self.data is not None and self.dataType == 'file' and isinstance(self.data, dict):
            filename, file_object, mime = self.data.get('attachment')

            file_object.seek(0)
            encoded_string = base64.b64encode(file_object.read())

            self.data = "{};{};{}".format(filename, mime, encoded_string.decode())

        return self




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import sys

import os
import warnings
import json
import magic
import requests

from thehive4py.auth import BasicAuth, BearerAuth
from thehive4py.models import CaseHelper, Version
from thehive4py.query import Parent, Id, And, Eq
from thehive4py.exceptions import *


class TheHiveApi:

    def __init__(self, url: str, principal: str, password=None, proxies={}, cert=True, organisation=None,
                 version=Version.THEHIVE_3.value):
        """
        Python API client for TheHive.

        Arguments:
            url (str): URL of Thehive instance, including the port. Ex: `http://myserver:9000`
            principal (str): The API key, or the username if basic authentication is used.
            password (str): The password for basic authentication or None. Defaults to None
            proxies (dict): The proxy configuration, would have `http` and `https` attributes. Defaults to {}
                ```python
                proxies: {
                    "http: "http://my_proxy:8080"
                    "https: "http://my_proxy:8080"
                }
                ```
            cert (bool): Wether or not to enable SSL certificate validation
            organisation (str): The name of the organisation against which api calls will be run. Defaults to None
            version (int): The version of TheHive instance. Defaults to 3


        ??? note "Examples"
            === "Basic"
                Example of simple usage: call TheHive APIs using an API key, without proxy, nor organisation

                ```python
                api = TheHiveApi('http://my_thehive:9000', 'my_api_key')
                ```

            === "Full options"
                Example using all the options: call TheHive APIs using an API key, with orgnisation, proxy and sst certificate

                ```python
                proxies = {
                    "http: "http://my_proxy:8080"
                    "https: "http://my_proxy:8080"
                }
                api = TheHiveApi('http://my_thehive:9000',
                    'my_api_key',
                    None,
                    proxies,
                    True,
                    'my-org',
                    version=Version.THEHIVE_3.value
                )
                ```
        """
        self.url = url
        self.principal = principal
        self.password = password
        self.proxies = proxies
        self.organisation = organisation

        if self.password is not None:
            self.auth = BasicAuth(self.principal, self.password, self.organisation)
        else:
            self.auth = BearerAuth(self.principal, self.organisation)

        self.cert = cert
        self.version = version

        # Create a CaseHelper instance
        self.case = CaseHelper(self)

    def __isVersion(self, version):
        return self.version is version

    def __find_rows(self, find_url, **attributes):
        """
        Private fuction that abstracts the calls to _search API

        Arguments:
            find_url: URL of the find api
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned
            nparent (int): Number of parent objects for a record to return. E.g. parent case(s) for a task or observable. The default used by TheHive when searching for these items is 10.

        Returns:
            response (requests.Response): Response object including a JSON array representing the list of searched records.

        Raises:
            TheHiveException
        """
        req = self.url + find_url

        # Add range and sort parameters
        params = {
            "range": attributes.get("range", "all"),
            "sort": attributes.get("sort", []),
            "nparent": attributes.get("nparent", 0)
        }

        # Add body
        data = {
            "query": attributes.get("query", {})
        }

        try:
            return requests.post(req, params=params, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("Error: {}".format(e))

    def do_patch(self, api_url, **attributes):
        return requests.patch(self.url + api_url, headers={'Content-Type': 'application/json'}, json=attributes,
                              proxies=self.proxies, auth=self.auth, verify=self.cert)

    def health(self):
        """
        Method to call the /api/health endpoint

        Returns:
            Response object resulting from the API call.

        Raises:
            TheHiveException: Generic exception if an error occurs
        """
        req = self.url + "/api/health"
        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("Error on retrieving health status: {}".format(e))

    def get_current_user(self):
        """
        Method to call the /api/current endpoint, returning the current authenticated user.

        Returns:
            response (requests.Response): Response object including a JSON description of the current user

        Raises:
            TheHiveException: Generic exception if an error occurs
        """

        req = self.url + "/api/user/current"
        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("Error on retrieving current user: {}".format(e))

    def create_case(self, case):

        """
        Create a case

        Arguments:
            case (Case): Instance of [Case][thehive4py.models.Case]

        Returns:
            response (requests.Response): Response object including a JSON description of a case

        Raises:
            CaseException: An error occured during case creation
        """

        req = self.url + "/api/case"
        data = case.jsonify(excludes=['id'])
        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseException("Case create error: {}".format(e))

    def update_case(self, case, fields=[]):
        """
        Update a case.

        Arguments:
            case (Case): Instance of [Case][thehive4py.models.Case] to update. The case's `id` determines which case to update.
            fields (Array): Optional parameter, an array of fields names, the ones we want to update

                Updatable fields are: [`title`, `description`, `severity`, `startDate`, `owner`, `flag`, `tlp`, `pap`, `tags`, `status`,
                `resolutionStatus`, `impactStatus`, `summary`, `endDate`, `metrics`, `customFields`]

        Returns:
            response (requests.Response): Response object including a JSON description of a case

        Raises:
            CaseException: An error occured during case creation
        """
        req = self.url + "/api/case/{}".format(case.id)

        # Choose which attributes to send
        update_keys = [
            'title', 'description', 'severity', 'startDate', 'owner', 'flag', 'tlp', 'pap', 'tags', 'status',
            'resolutionStatus', 'impactStatus', 'summary', 'endDate', 'metrics', 'customFields'
        ]
        data = {k: v for k, v in case.__dict__.items() if (len(fields) > 0 and k in fields) or (len(fields) == 0 and k in update_keys)}
        try:
            return requests.patch(req, headers={'Content-Type': 'application/json'}, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseException("Case update error: {}".format(e))

    def create_case_task(self, case_id, case_task):

        """
        Create a case task

        Arguments:
            case_id: Case identifier
            case_task: Instance of [CaseTask][thehive4py.models.CaseTask]

        Returns:
            response (requests.Response): Response object including a JSON description of a case task

        Raises:
            CaseTaskException: An error occured during case task creation

        """

        req = self.url + "/api/case/{}/task".format(case_id)
        data = case_task.jsonify(excludes=['id'])

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseTaskException("Case task create error: {}".format(e))

    def update_case_task(self, task, fields=[]):
        """
        Update a case task

        Arguments:
            task (CaseTask): Instance of [CaseTask][thehive4py.models.CaseTask]
            fields (array): Arry of strings representing CaseTask properties to be updated
               
                Updatable fields are: [`title`, `description`, `status`, `order`, `user`, `owner`, `flag`, `endDate`]

        Returns:
            response (requests.Response): Response object including a JSON description of a case task

        Raises:
            CaseTaskException: An error occured during case task creation
        """

        req = self.url + "/api/case/task/{}".format(task.id)

        # Choose which attributes to send
        update_keys = [
            'title', 'description', 'status', 'order', 'user', 'owner', 'flag', 'endDate'
        ]

        data = {k: v for k, v in task.__dict__.items() if (
            len(fields) > 0 and k in fields) or (len(fields) == 0 and k in update_keys)}

        try:
            return requests.patch(req, headers={'Content-Type': 'application/json'}, json=data,
                                  proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseTaskException("Case task update error: {}".format(e))

    def delete_case_task(self, task_id):
        """
        Deletes a TheHive case task.

        Arguments:
            task_id (str): Id of the task to delete

        Returns:
            response (requests.Response): Response object including the updated task

        Raises:
            CaseException: An error occured during case deletion
        """
        req = self.url + "/api/case/task/{}".format(task_id)
        try:
            return requests.patch(req, headers={'Content-Type': 'application/json'}, json={'status': 'Cancel'},
                                   proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseTaskException("Case task deletion error: {}".format(e))

    def create_task_log(self, task_id, case_task_log):

        """
        Create a task log either with an attachement or just with a log message.

        Arguments:
            task_id (str): Task identifier
            case_task_log (CaseTaskLocg): Instance of [CaseTaskLog][thehive4py.models.CaseTaskLog]

        Returns:
            response (requests.Response): Response object including a JSON description of a case

        Raises:
            CaseException: An error occured during case creation
        """

        req = self.url + "/api/case/task/{}/log".format(task_id)
        data = {'_json': json.dumps({"message": case_task_log.message})}

        if case_task_log.file:
            f = case_task_log.attachment

            try:
                return requests.post(req, data=data, files=f, proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise CaseTaskException("Case task log create error: {}".format(e))
        else:
            try:
                return requests.post(req, headers={'Content-Type': 'application/json'}, data=json.dumps({'message':case_task_log.message}), proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise CaseTaskException("Case task log create error: {}".format(e))

    def create_case_observable(self, case_id, case_observable):

        """
        Create a case observable

        Arguments:
            case_id (str): Case identifier
            case_observable (CaseObservable): Instance of [CaseObservable][thehive4py.models.CaseObservable]

        Returns:
            response (requests.Response): Response object including a JSON description of a case observable

        Raises:
            CaseObservableException: An error occured during case observable creation
        """

        req = self.url + "/api/case/{}/artifact".format(case_id)

        if case_observable.dataType == 'file':
            try:

                data = {
                    "dataType": case_observable.dataType,
                    "message": case_observable.message,
                    "tlp": case_observable.tlp,
                    "tags": case_observable.tags,
                    "ioc": case_observable.ioc,
                    "sighted": case_observable.sighted,
                    "ignoreSimilarity": case_observable.ignoreSimilarity
                }

                # Exclude ignoreSimilarity field for TheHive 3
                if self.__isVersion(Version.THEHIVE_3.value):
                    data.pop('ignoreSimilarity', None)

                data = {"_json": json.dumps(data)}
                return requests.post(req, data=data, files=case_observable.data[0], proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise CaseObservableException("Case observable create error: {}".format(e))
        else:
            try:
                to_exclude = ['id']

                # Exclude ignoreSimilarity field for TheHive 3
                if self.__isVersion(Version.THEHIVE_3.value):
                    to_exclude.append('ignoreSimilarity')

                data = case_observable.jsonify(excludes=to_exclude)

                return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise CaseObservableException("Case observable create error: {}".format(e))

    def delete_case_observable(self, observable_id):
        """
        Deletes a TheHive case observable.

        Arguments:
            observable_id (str): Id of the observable to delete

        Returns:
            response (requests.Response): Response object including true or false based on the action's success

        Raises:
            CaseObservableException: An error occured during case observable deletion
        """
        req = self.url + "/api/case/artifact/{}".format(observable_id)

        try:
            return requests.delete(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Case observable deletion error: {}".format(e))

    def update_case_observable(self, observable_id, case_observable, fields=[]):

        """
        Update an existing case observable

        Arguments:
            observable_id: Observable identifier
            case_observable (CaseObservable): Instance of [CaseObservable][thehive4py.models.CaseObservable]
            fields (Array): Optional parameter, an array of fields names, the ones we want to update.

                Updatable fields are: [`tlp`, `ioc`, `sighted`, `tags`, `message`, `ignoreSimilarity`]

        Returns:
            response (requests.Response): Response object including a JSON description of the updated case observable

        Raises:
            CaseObservableException: An error occured during case observable update
        """

        req = self.url + "/api/case/artifact/{}".format(observable_id)

        update_keys = ['message', 'tlp', 'tags', 'ioc', 'sighted', 'ignoreSimilarity']

        data = {k: v for k, v in case_observable.__dict__.items() if (
                len(fields) > 0 and k in fields) or (len(fields) == 0 and k in update_keys)}

        # Exclude ignoreSimilarity field for TheHive 3
        if self.__isVersion(Version.THEHIVE_3.value):
            data.pop('ignoreSimilarity', None)

        try:
            return requests.patch(req, headers={'Content-Type': 'application/json'}, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Case observable update error: {}".format(e))

    def get_case(self, case_id):
        """
        Get a case by id

        Arguments:
            case_id (str): Case identifier

        Returns:
            response (requests.Response): Response object including a JSON description of the case.

        Raises:
            CaseException: An error occured during case fetch
        """

        req = self.url + "/api/case/{}".format(case_id)

        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseException("Case fetch error: {}".format(e))

    def find_cases(self, **attributes):
        """
        Find cases using sort, pagination and a query

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of cases.

        Raises:
            CaseException: An error occured during case search
        """
        return self.__find_rows("/api/case/_search", **attributes)

    def delete_case(self, case_id, force=False):
        """
        Deletes a TheHive case. Unless force is set to True the case is 'soft deleted' (status set to deleted).

        Arguments:
            case_id (str): Id of the case to delete
            force (bool): True to physically delete the case, False to mark the case as deleted

        Returns:
            response (requests.Response): Response object including true or false based on the action's success

        Raises:
            CaseException: An error occured during case deletion
        """
        req = self.url + "/api/case/{}".format(case_id)
        if force:
            req += '/force'
        try:
            return requests.delete(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseException("Case deletion error: {}".format(e))

    def find_first(self, **attributes):
        """
        Find cases and return just the first record

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order

        Returns:
            response (dict): A dict object describing the first case resulting from the query and sort options.

        Raises:
            CaseException: An error occured during case search
        """
        attributes['range'] = '0-1'

        try:
            return self.find_cases(**attributes).json()[0]
        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Case search error: {}".format(e))

    def get_case_observable(self, observable_id):

        """
        Get a case observable by its id

        Arguments:
            observable_id (str): Case observable identifier

        Returns:
            response (requests.Response): Response object including a JSON representation of the case observable

        Raises:
            CaseObservableException: An error occured during case observable fetch
        """

        req = self.url + "/api/case/artifact/{}".format(observable_id)

        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Case observable search error: {}".format(e))

    def get_case_observables(self, case_id, **attributes):

        """
        Find observables of a given case identified by its id

        Arguments:
            case_id (str): Id of the case
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of case observable.

        Raises:
            CaseObservableException: An error occured during case observable search
        """

        req = self.url + "/api/case/artifact/_search"

        # Add range and sort parameters
        params = {
            "range": attributes.get("range", "all"),
            "sort": attributes.get("sort", [])
        }

        # Add body
        parent_criteria = Parent('case', Id(case_id))

        # Append the custom query if specified
        if "query" in attributes:
            criteria = And(parent_criteria, attributes["query"])
        else:
            criteria = parent_criteria

        data = {
            "query": criteria
        }

        try:
            return requests.post(req, params=params, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Case observables search error: {}".format(e))

    def find_observables(self, **attributes):
        """
        Find observables using sort, pagination and a query

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of observables.

        Raises:
            ObservableException: An error occured during observable search
        """

        return self.__find_rows("/api/case/artifact/_search", **attributes)

    def get_case_tasks(self, case_id, **attributes):
        """
        Find tasks of a given case identified by its id

        Arguments:
            case_id (str): Id of the case
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of case task.

        Raises:
            CaseTaskException: An error occured during case task search
        """

        req = self.url + "/api/case/task/_search"

        # Add range and sort parameters
        params = {
            "range": attributes.get("range", "all"),
            "sort": attributes.get("sort", [])
        }

        # Add body
        parent_criteria = Parent('case', Id(case_id))

        # Append the custom query if specified
        if "query" in attributes:
            criteria = And(parent_criteria, attributes["query"])
        else:
            criteria = parent_criteria

        data = {
            "query": criteria
        }

        try:
            return requests.post(req, params=params, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseTaskException("Case tasks search error: {}".format(e))

    def get_linked_cases(self, case_id):
        """
        Find related cases of a given case identified by its id

        Arguments:
            case_id (str): Id of the case

        Returns:
            response (requests.Response): Response object including a JSON array of related cases.

        Raises:
            CaseException: An error occured during case links fetch
        """
        req = self.url + "/api/case/{}/links".format(case_id)

        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseException("Linked cases fetch error: {}".format(e))

    def find_case_templates(self, **attributes):
        """
        Find case templates using a query, sort and pagination

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of case templates

        Raises:
            TheHiveException: An error occured during case template search
        """
        return self.__find_rows("/api/case/template/_search", **attributes)

    def get_case_template(self, name):

        """
        Get a case template by its name

        Arguments:
            name (str): Case template's name

        Returns:
            response (requests.Response): Response object including a JSON representation of the case template

        Raises:
            CaseTemplateException: An error occured during case template fetch
        """

        req = self.url + "/api/case/template/_search"

        if self.__isVersion(Version.THEHIVE_3.value):
            query = And(Eq("name", name), Eq("status", "Ok"))
        else:
            query = Eq("name", name)

        data = {
            "query": query
        }

        try:
            response = requests.post(req, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
            json_response = response.json()

            if response.status_code == 200 and len(json_response) > 0:
                return response.json()[0]
            else:
                raise CaseTemplateException("Case template fetch error: Unable to find case template {}".format(name))
        except requests.exceptions.RequestException as e:
            raise CaseTemplateException("Case template fetch error: {}".format(e))

    def create_case_template(self, case_template):
        """
        Create a case template

        Arguments:
            case_template (CaseTemplate): Instance of [CaseTemplate][thehive4py.models.CaseTemplate]

        Returns:
            response (requests.Response): Response object including a JSON representation of the case template

        Raises:
            CaseTemplateException: An error occured during case template creation
        """

        req = self.url + "/api/case/template"
        data = case_template.jsonify(excludes=['id'])

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseTemplateException("Case template create error: {}".format(e))

    def _check_if_custom_field_exists(self, custom_field):
        data = {
            'key': 'reference',
            'value': custom_field.reference
        }
        req = self.url + "/api/list/custom_fields/_exists"
        response = requests.post(req, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        return response.json().get('found', 'False')

    def create_custom_field(self, custom_field):
        """
        Create a custom field

        Arguments:
            custom_field (CustomField): Instance of [CustomField][thehive4py.models.CustomField]

        Returns:
            response (requests.Response): Response object including a JSON representation of the case template

        Raises:
            CustomFieldException: Custom field already exists
            CustomFieldException: An error occured during custom field creation

        !!! Warning
            This function is available only for TheHive 3
        """

        if self._check_if_custom_field_exists(custom_field):
            raise CustomFieldException('Field with reference "{}" already exists'.format(custom_field.reference))

        data = {
            "value": {
                "name": custom_field.name,
                "reference": custom_field.reference,
                "description": custom_field.description,
                "type": custom_field.type,
                "options": custom_field.options,
                "mandatory": custom_field.mandatory
                }
            }
        req = self.url + "/api/list/custom_fields"
        try:
            return requests.post(req, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CustomFieldException("Custom field create error: {}".format(e))

    def get_case_task(self, task_id):
        """
        Get a case task by its id

        Arguments:
            task_id (str): Case task identifier

        Returns:
            response (requests.Response): Response object including a JSON representation of the case task

        Raises:
            CaseTaskException: An error occured during case task fetch
        """

        req = self.url + "/api/case/task/{}".format(task_id)
        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseTaskException("Case task logs search error: {}".format(e))

    def get_task_log(self, log_id):
        """
        Get a case task log by its id

        Arguments:
            log_id (str): Case task log identifier

        Returns:
            response (requests.Response): Response object including a JSON representation of the case task log

        Raises:
            CaseTaskException: An error occured during case task log fetch
        """

        if self.__isVersion(Version.THEHIVE_3.value):
            req = self.url + "/api/case/task/log/{}".format(log_id)
            try:
                return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise CaseTaskLogException("Case task log fetch error: {}".format(e))
        else:
            req = self.url + "/api/v1/query"

            data = {
                "query": [
                    {"_name": "getLog", "idOrName": log_id}
                ]
            }
            try:
                return requests.post(req, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise CaseTaskLogException("Case task log fetch error: {}".format(e))
            #'{"query": [{"_name": "getLog", "idOrName": "~40976560"}]}'

    def get_task_logs(self, task_id, **attributes):
        """
        Get logs of a case task by its id

        Arguments:
            task_id (str): Case task identifier
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array representing a list of case task logs

        Raises:
            CaseTaskException: An error occured during case task log search
        """

        req = self.url + "/api/case/task/log/_search"

        # Add range and sort parameters
        params = {
            "range": attributes.get("range", "all"),
            "sort": attributes.get("sort", [])
        }

        # Add body
        parent_criteria = Parent('case_task', Id(task_id))

        # Append the custom query if specified
        if "query" in attributes:
            criteria = And(parent_criteria, attributes["query"])
        else:
            criteria = parent_criteria

        data = {
            "query": criteria
        }

        return self.find_task_logs(query=criteria, **params)

    def create_alert(self, alert):

        """
        Create an alert. Supports adding observables and custom fields

        Arguments:
            alert (Alert): Instance of [Alert][thehive4py.models.Alert]

        Returns:
            response (requests.Response): Response object including a JSON array representing a list of case task logs

        Raises:
            AlertException: An error occured during alert creation
        """

        req = self.url + "/api/alert"

        to_exclude = ['id']

        # Exclude PAP field for TheHive 3
        if self.__isVersion(Version.THEHIVE_3.value):
            to_exclude.append('pap')
            to_exclude.append('externalLink')

        data = alert.jsonify(excludes=to_exclude)

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Alert create error: {}".format(e))

    def mark_alert_as_read(self, alert_id):
        """
        Mark an alert as read. This sets the status of the alert to `Ignored` if it's not yet promoted to a case.

        Arguments:
            alert_id (str): Id of the alert

        Returns:
            response (requests.Response): Response object including a JSON representation of the alert

        Raises:
            AlertException: An error occured during alert update
        """
        req = self.url + "/api/alert/{}/markAsRead".format(alert_id)

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Mark alert as read error: {}".format(e))

    def mark_alert_as_unread(self, alert_id):
        """
        Mark an alert as unread. This sets the status of the alert to `New` if it's not yet promoted to a case.

        Arguments:
            alert_id (str): Id of the alert

        Returns:
            response (requests.Response): Response object including a JSON representation of the alert

        Raises:
            AlertException: An error occured during alert update
        """
        req = self.url + "/api/alert/{}/markAsUnread".format(alert_id)

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Mark alert as unread error: {}".format(e))

    def merge_alert_into_case(self, alert_id, case_id):
        """
        Merge alert into existing case.
        :param alert_id: The ID of the alert to merge.
        :param case_id: The ID of the case where to merge alert
        :return:
        """
        req = self.url + "/api/alert/{}/merge/{}".format(alert_id, case_id)

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, json={}, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Merge alert to case error: {}".format(e))

    def update_alert(self, alert_id, alert, fields=[]):
        """
        Update an alert completely or using specified fields

        Arguments:
            alert_id (str): Id of the alert
            alert (Alert): Instance of [Alert][thehive4py.models.Alert]
            fields (Array): Optional parameter, an array of field names, the ones we want to update

                Updatable fields are: [`tlp`, `severity`, `tags`, `caseTemplate`, `title`, `description`, `customFields`]

        Returns:
            response (requests.Response): Response object including a JSON representation of the alert

        Raises:
            AlertException: An error occured during alert update
        """
        req = self.url + "/api/alert/{}".format(alert_id)

        # update only the alert attributes that are not read-only
        update_keys = ['tlp', 'pap', 'severity', 'tags', 'caseTemplate', 'title', 'description', 'customFields',
                       'artifacts', 'follow']

        data = {k: v for k, v in alert.__dict__.items() if (
                len(fields) > 0 and k in fields) or (len(fields) == 0 and k in update_keys)}

        if 'artifacts' in data:
            data['artifacts'] = [a.__dict__ for a in alert.artifacts]
            # data['artifacts'] = [{k: v for k, v in a.__dict__.items()} for a in alert.artifacts]

        # Exclude PAP field for TheHive 3
        if self.__isVersion(Version.THEHIVE_3.value):
            data.pop('pap', None)
            data.pop('externalLink', None)

        try:
            return requests.patch(req, headers={'Content-Type': 'application/json'}, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Alert update error: {}".format(e))

    def get_alert(self, alert_id, similar_cases=False):
        """
        Get an alert by its id

        Arguments:
            alert_id (str): Id of the alert
            similar_cases (bool): True if similar cases should be retrieved (Default False)

        Returns:
            response (requests.Response): Response object including a JSON representation of the alert

        Raises:
            AlertException: An error occured during alert update
        """

        req = self.url + "/api/alert/{}".format(alert_id)

        params = {}

        if similar_cases:
            params = {
                "similarity": int(similar_cases)
            }

        try:
            return requests.get(req, proxies=self.proxies, params=params, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Alert fetch error: {}".format(e))

    def find_alerts(self, **attributes):
        """
        Find alerts using sort, pagination and a query

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of alerts.

        Raises:
            AlertException: An error occured during alert search
        """

        return self.__find_rows("/api/alert/_search", **attributes)

    def delete_alert(self, alert_id):
        """
        Deletes a TheHive alert.

        Arguments:
            alert_id (str): Id of the alert to delete

        Returns:
            response (requests.Response): Response object including true or false based on the action's success

        Raises:
            AlertException: An error occured during alert deletion

        !!! Warning
            TheHive 3: Deleting alert requires `admin` role
            TheHive 4: Deleting alert requires a role including `manageAlert` permissing
        """
        
        req = self.url + "/api/alert/{}".format(alert_id)
        params = {
            "force": 1
        }
        try:
            return requests.delete(req, params=params, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertException("Alert deletion error: {}".format(e))

    def update_case_observables(self, observable, fields=[]):
        """
        [DEPRECATED] Update a case observable

        Arguments:
            observable (CaseObservable): Instance of [CaseObservable][thehive4py.models.CaseObservable] to update. 
                The observable's `id` determines which case to update.
            fields (Array): Optional parameter, an array of fields names, the ones we want to update.
                
                Updatable fields are: [`tlp`, `ioc`, `sighted`, `tags`, `message`, `ignoreSimilarity`]

        Returns:
            response (requests.Response): Response object including a JSON description of a case observable

        Raises:
            CaseObservableException: An error occured during observable update
        """
        return self.update_case_observable(observable.id, observable, fields=fields)

    def promote_alert_to_case(self, alert_id, case_template=None):
        """
        Create a new case from an alert, with an optional case template

        Arguments:
            alert_id (str): Id of the alert
            case_template (str): Case template name to apply when creating the cas
            
        Returns:
            response (requests.Response): Response object including a JSON representation of the alert

        Raises:
            AlertException: An error occured during alert promotion
        """

        req = self.url + "/api/alert/{}/createCase".format(alert_id)

        try:
            return requests.post(req, headers={'Content-Type': 'application/json'},
                                 proxies=self.proxies, auth=self.auth,
                                 verify=self.cert, data=json.dumps({"caseTemplate": case_template}))

        except requests.exceptions.RequestException as the_exception:
            raise AlertException("Couldn't promote alert to case: {}".format(the_exception))

        return None

    def run_analyzer(self, cortex_id, artifact_id, analyzer_id):
        """
        Create a new case from an alert, with an optional case template

        Arguments:
            cortex_id: identifier of the Cortex server
            artifact_id: identifier of the artifact as found with an artifact search
            analyzer_id: name of the analyzer used by the job
            
        Returns:
            response (requests.Response): Response object including a JSON representation of the analysis job

        Raises:
            TheHiveException: An error occured during job creation
        """

        req = self.url + "/api/connector/cortex/job"

        try:
            data = json.dumps({ "cortexId": cortex_id,
                "artifactId": artifact_id,
                "analyzerId": analyzer_id
                })
            return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("Analyzer run error: {}".format(e))

    def get_job_status(self, job_id):
        """
        Get the job status previously launched

        Arguments:
            job_id: Analysis job ID
            
            
        Returns:
            response (requests.Response): Response object including a JSON representation of the analysis job

        Raises:
            TheHiveException: An error occured during job fetch
        """

        req = self.url + "/api/connector/cortex/job/{job_id}?nstats=true".format(job_id = job_id)

        try:
            return requests.get(req, headers={'Content-Type': 'application/json'}, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("Analyzer run error: {}".format(e))

    def find_tasks(self, **attributes):
        """
        Find case tasks using sort, pagination and a query

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array of case tasks.

        Raises:
            AlertException: An error occured during case task search
        """

        return self.__find_rows("/api/case/task/_search", **attributes)

    def find_task_logs(self, **attributes):
        """
        Find task logs using sort, pagination and a query

        Arguments:
            query (dict): A query object, defined in JSON format or using utiliy methods from thehive4py.query module
            sort (Array): List of fields to sort the result with. Prefix the field name with `-` for descending order
                and `+` for ascending order
            range (str): A range describing the number of rows to be returned

        Returns:
            response (requests.Response): Response object including a JSON array representing a list of case task logs

        Raises:
            CaseTaskException: An error occured during case task log search
        """

        return self.__find_rows("/api/case/task/log/_search", **attributes)

    def export_to_misp(self, misp_id, case_id):
        """
        Export selected IOCs of a case as an event to a MISP instance 
        This function triggers the same action triggered when the "Share" button on the TheHive GUI is clicked

        Arguments:
            misp_id: identifier of the MISP server
            case_id (str): Id of the case
            
        Returns:
            response (requests.Response): Response object including a JSON representation of the exported event

        Raises:
            TheHiveException: An error occured during the export operation
        """

        req = self.url + "/api/connector/misp/export/{0}/{1}".format(case_id, misp_id)
        try:
            return requests.post(req, headers={'Content-Type': 'application/json'}, proxies=self.proxies,
                                 json={}, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("MISP export error: {}".format(e))

    def download_attachment(self, attachment_id, filename="attachment", archive=False):
        """
        Get the content of an attachement object by ID

        Arguments:
            attachment_id: identifier of the attachment object
            filename (str): name of the downloaded file
            archive (bool): set to `True` to zip and password protect the downloaded file

        Returns:
            response (requests.Response): Response object including a the attachment content as bytes

        Raises:
            TheHiveException: An error occured during the attachment download
        """
        if archive is True:
            req = self.url + "/api/datastorezip/{}?name{}".format(attachment_id, filename)
        else:
            req = self.url + "/api/datastore/{}?name={}".format(attachment_id, filename)

        try:
            return requests.get(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise TheHiveException("Error on retrieving attachment {}: {}".format(attachment_id, e))

    def download_task_log_attachment(self, task_log_id, archive=False):
        """
        Get the content of the attachement object of a task log

        Arguments:
            task_log_id: identifier of the task log object
            archive (bool): set to `True` to zip and password protect the downloaded file

        Returns:
            response (requests.Response): Response object including a the attachment content as bytes

        Raises:
            CaseTaskLogException: If the task log doesn't have an attachment
            TheHiveException: An error occured during the attachment download
        """
        try:
            # Get the task log by id
            response = self.get_task_log(task_log_id)

            # Check if it has an attachment
            if self.__isVersion(Version.THEHIVE_3.value):
                log = response.json()
            else:
                log = response.json()[0]

            if 'attachment' in log:
                attachment = log['attachment']
                return self.download_attachment(attachment['id'], filename=attachment['name'], archive=archive)
            else:
                raise CaseTaskLogException("Task log {} doesn't have an attachment".format(task_log_id))

        except requests.exceptions.RequestException as e:
            raise CaseTaskLogException("Error on retrieving attachment of task log {}: {}".format(task_log_id, e))

    def download_observable_attachment(self, observable_id, archive=True):
        """
        Get the content of the attachement object of a file observable

        Arguments:
            observable_id: identifier of the case observable object
            archive (bool): set to `False` to disable zip and password protection of the downloaded file

        Returns:
            response (requests.Response): Response object including a the attachment content as bytes

        Raises:
            CaseObservableException: If the observable is not a file
            TheHiveException: An error occured during the attachment download
        """
        try:
            # Get the observable by id
            response = self.get_case_observable(observable_id)

            # Check if it has an attachment
            observable = response.json()

            if 'attachment' in observable:
                attachment = observable['attachment']
                return self.download_attachment(attachment['id'], filename=attachment['name'], archive=True)
            else:
                raise CaseObservableException("Observable {} doesn't have an attachment".format(observable_id))

        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Error on retrieving attachment of case observable {}: {}".format(observable_id, e))

    def create_alert_artifact(self, alert_id, alert_artifact):

        """
        Create an alert artifact

        Arguments:
            alert_id (str): Alert identifier
            alert_artifact (AlertArtifact): Instance of [AlertArtifact][thehive4py.models.AlertArtifact]

        Returns:
            response (requests.Response): Response object including a JSON description of an alert artifact

        Raises:
            AlertArtifactException: An error occured during alert artifact creation

        !!! Warning
            This function is available in TheHive 4 ONLY
        """

        if self.__isVersion(Version.THEHIVE_3.value):
            raise AlertArtifactException("This function is available in TheHive 4 ONLY")

        req = self.url + "/api/alert/{}/artifact".format(alert_id)

        if alert_artifact.dataType == 'file':
            try:
                fields = ["dataType", "message", "tlp", "tags", "ioc", "sighted", "ignoreSimilarity"]
                data = {k: v for k, v in alert_artifact.__dict__.items() if k in fields}

                data = {"_json": json.dumps(data)}
                return requests.post(req, data=data, files=alert_artifact.data, proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise AlertArtifactException("Alert artifact create error: {}".format(e))
        else:
            try:
                data = alert_artifact.jsonify(excludes=['id'])

                return requests.post(req, headers={'Content-Type': 'application/json'}, data=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
            except requests.exceptions.RequestException as e:
                raise AlertArtifactException("Alert artifact create error: {}".format(e))

    def delete_alert_artifact(self, artifact_id):
        """
        Deletes a TheHive alert artifact.

        Arguments:
            artifact_id (str): Id of the artifact to delete

        Returns:
            response (requests.Response): Response object including true or false based on the action's success

        Raises:
            AlertArtifactException: An error occured during alert artifact deletion

        !!! Warning
            This function is available in TheHive 4 ONLY
        """

        if self.__isVersion(Version.THEHIVE_3.value):
            raise AlertArtifactException("This function is available in TheHive 4 ONLY")

        req = self.url + "/api/alert/artifact/{}".format(artifact_id)

        try:
            return requests.delete(req, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise AlertArtifactException("Alert artifact deletion error: {}".format(e))

    def update_alert_artifact(self, artifact_id, alert_artifact, fields=[]):

        """
        Update an existing alert artifact

        Arguments:
            artifact_id: Artifact identifier
            alert_artifact (AlertArtifact): Instance of [AlertArtifact][thehive4py.models.AlertArtifact]
            fields (Array): Optional parameter, an array of fields names, the ones we want to update.

                Updatable fields are: [`tlp`, `ioc`, `sighted`, `tags`, `message`, `ignoreSimilarity`]

        Returns:
            response (requests.Response): Response object including a JSON description of the updated alert artifact

        Raises:
            AlertArtifactException: An error occured during alert artifact update

        !!! Warning
            This function is available in TheHive 4 ONLY
        """

        if self.__isVersion(Version.THEHIVE_3.value):
            raise AlertArtifactException("This function is available in TheHive 4 ONLY")

        req = self.url + "/api/alert/artifact/{}".format(artifact_id)

        update_keys = ['message', 'tlp', 'tags', 'ioc', 'sighted', 'ignoreSimilarity']

        data = {k: v for k, v in alert_artifact.__dict__.items() if (
                len(fields) > 0 and k in fields) or (len(fields) == 0 and k in update_keys)}

        try:
            return requests.patch(req, headers={'Content-Type': 'application/json'}, json=data, proxies=self.proxies, auth=self.auth, verify=self.cert)
        except requests.exceptions.RequestException as e:
            raise CaseObservableException("Case observable update error: {}".format(e))



import time
from thehive4py.models import CustomFieldHelper


def test_boolean():
    custom_fields = CustomFieldHelper() \
        .add_boolean('boolean-field', True) \
        .build()

    assert custom_fields is not None
    assert 'boolean-field' in custom_fields
    assert 'boolean' in custom_fields['boolean-field']
    assert 'order' in custom_fields['boolean-field']

    assert custom_fields['boolean-field']['boolean'] is True
    assert custom_fields['boolean-field']['order'] == 0


def test_date():
    date = int(time.time())*1000

    custom_fields = CustomFieldHelper() \
        .add_date('date-field', date) \
        .build()

    assert custom_fields is not None
    assert 'date-field' in custom_fields
    assert 'date' in custom_fields['date-field']
    assert 'order' in custom_fields['date-field']

    assert custom_fields['date-field']['date'] == date
    assert custom_fields['date-field']['order'] == 0


def test_string():
    custom_fields = CustomFieldHelper() \
        .add_string('string-field', 'thehive') \
        .build()

    assert custom_fields is not None
    assert 'string-field' in custom_fields
    assert 'string' in custom_fields['string-field']
    assert 'order' in custom_fields['string-field']

    assert custom_fields['string-field']['string'] == 'thehive'
    assert custom_fields['string-field']['order'] == 0


def test_number():
    custom_fields = CustomFieldHelper() \
        .add_number('number-field', 100) \
        .build()

    assert custom_fields is not None
    assert 'number-field' in custom_fields
    assert 'number' in custom_fields['number-field']
    assert 'order' in custom_fields['number-field']

    assert custom_fields['number-field']['number'] == 100
    assert custom_fields['number-field']['order'] == 0


def test_intger():
    custom_fields = CustomFieldHelper() \
        .add_integer('integer-field', 100) \
        .build()

    assert custom_fields is not None
    assert 'integer-field' in custom_fields
    assert 'integer' in custom_fields['integer-field']
    assert 'order' in custom_fields['integer-field']

    assert custom_fields['integer-field']['integer'] == 100
    assert custom_fields['integer-field']['order'] == 0


def test_intger():
    custom_fields = CustomFieldHelper() \
        .add_float('float-field', 1.0) \
        .build()

    assert custom_fields is not None
    assert 'float-field' in custom_fields
    assert 'float' in custom_fields['float-field']
    assert 'order' in custom_fields['float-field']

    assert custom_fields['float-field']['float'] == 1.0
    assert custom_fields['float-field']['order'] == 0


def test_order():
    custom_fields = CustomFieldHelper()\
        .add_boolean('booleanField', True)\
        .add_string('businessImpact', 'HIGH')\
        .add_date('occurDate', int(time.time())*1000)\
        .add_number('cvss', 9) \
        .add_integer('csirts', 2) \
        .add_float('hitRate', 50.2) \
        .build()

    assert custom_fields is not None
    assert 'booleanField' in custom_fields
    assert 'businessImpact' in custom_fields
    assert 'occurDate' in custom_fields
    assert 'cvss' in custom_fields
    assert 'csirts' in custom_fields
    assert 'hitRate' in custom_fields

    assert custom_fields['booleanField']['order'] == 0
    assert custom_fields['businessImpact']['order'] == 1
    assert custom_fields['occurDate']['order'] == 2
    assert custom_fields['cvss']['order'] == 3
    assert custom_fields['csirts']['order'] == 4
    assert custom_fields['hitRate']['order'] == 5





try:
    import mock
except ImportError:
    import unittest.mock as mock

from thehive4py.api import TheHiveApi


@mock.patch('thehive4py.api.requests.get')
def test_get_case(mock_get):
    thehive = TheHiveApi('http://127.0.0.1:9000', 'API_KEY')

    test_id = 'AV55EOIsPQ_zDQrlj4a9'
    test_json = {
        '_type': 'case',
        'caseId': 5,
        'createdAt': 1505269703195,
        'createdBy': 'username',
        'customFields': {},
        'description': 'test description',
        'flag': False,
        'id': test_id,
        'metrics': {},
        'owner': 'username',
        'severity': 2,
        'startDate': 1505269703000,
        'status': 'Open',
        'tags': [],
        'title': 'test case',
        'tlp': 2,
        'user': 'username'
    }

    mock_response = mock.Mock()
    mock_response.json.return_value = test_json
    mock_response.status_code = 200
    mock_get.return_value = mock_response

    case = thehive.case(test_id)

    assert mock_response.json.call_count == 1
    assert case.id == test_id




#!/usr/bin/python
# -*- coding: utf-8 -*-
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='thehive4py',
    version='1.8.2',
    description='Python API client for TheHive.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='TheHive-Project',
    author_email='support@thehive-project.org',
    maintainer='TheHive-Project',
    url='https://github.com/TheHive-Project/Thehive4py',
    license='AGPL-V3',
    packages=['thehive4py'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    include_package_data=True,
    install_requires=['future', 'requests', 'python-magic']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests
from requests.auth import AuthBase


class BasicAuth(AuthBase):
    """
    A custom basic authentication class for requests, that takes into account the organisation header
    """
    def __init__(self, username, password, organisation=None):
        """
        Constructor

        Arguments:
            username (str): The username to use for the authentication.
            password (str): The password to use for the authentication.
            organisation (str): The organisation to use.
        """
        self.username = username
        self.password = password
        self.organisation = organisation

    def __call__(self, req):
        req.headers['Authorization'] = requests.auth._basic_auth_str(self.username, self.password)

        if self.organisation is not None:
            req.headers['X-Organisation'] = self.organisation

        return req


class BearerAuth(AuthBase):
    """
    A custom authentication class for requests, relying on API key (Bearer authorization header), 
    and taking into account the organisation header
    """
    def __init__(self, api_key, organisation=None):
        """
        Constructor

        Arguments:
            api_key (str): The API Key to use for the authentication
            organisation (str): The organisation to use.
        """    
        self.api_key = api_key
        self.organisation = organisation

    def __call__(self, req):
        req.headers['Authorization'] = 'Bearer {}'.format(self.api_key)

        if self.organisation is not None:
            req.headers['X-Organisation'] = self.organisation

        return req
